"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("mz/fs");
const path = require("path");
const symlinkDir = require("symlink-dir");
const exists = require("path-exists");
const pnpm_logger_1 = require("pnpm-logger");
const R = require("ramda");
const linkBins_1 = require("./linkBins");
const resolvePeers_1 = require("./resolvePeers");
const logInstallStatus_1 = require("../logging/logInstallStatus");
const updateShrinkwrap_1 = require("./updateShrinkwrap");
const dp = require("dependency-path");
const pnpm_shrinkwrap_1 = require("pnpm-shrinkwrap");
const removeOrphanPkgs_1 = require("../api/removeOrphanPkgs");
const linkIndexedDir_1 = require("../fs/linkIndexedDir");
const ncpCB = require("ncp");
const thenify = require("thenify");
const Rx = require("@reactivex/rxjs");
const shrinkwrap_1 = require("../fs/shrinkwrap");
const ncp = thenify(ncpCB);
function default_1(rootNodeId$, tree, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        pnpm_logger_1.default.info(`Creating dependency tree`);
        const resolvePeersResult = resolvePeers_1.default(tree, rootNodeId$, opts.topParent$, opts.independentLeaves, opts.baseNodeModules, {
            nonDevPackageIds: opts.nonDevPackageIds,
            nonOptionalPackageIds: opts.nonOptionalPackageIds,
        });
        const resolvedNode$ = resolvePeersResult.resolvedNode$;
        const rootResolvedNode$ = resolvePeersResult.rootResolvedNode$;
        resolvedNode$.subscribe({
            error: () => { },
            next: () => { },
            complete: () => pnpm_logger_1.stageLogger.debug('resolution_done'),
        });
        const depShr$ = updateShrinkwrap_1.default(resolvedNode$, opts.shrinkwrap, opts.pkg);
        const filterOpts = {
            noDev: opts.production,
            noOptional: !opts.optional,
            skipped: opts.skipped,
        };
        const updatedPkgsAbsolutePaths$ = linkNewPackages(filterShrinkwrap(opts.privateShrinkwrap, filterOpts), depShr$, opts, opts.shrinkwrap.registry);
        const updatedPkgsAbsolutePaths = yield updatedPkgsAbsolutePaths$
            .toArray()
            .toPromise();
        const shrPackages = opts.shrinkwrap.packages || {};
        yield depShr$.forEach(depShr => {
            shrPackages[depShr.dependencyPath] = depShr.snapshot;
        });
        opts.shrinkwrap.packages = shrPackages;
        const rootResolvedNodes = yield rootResolvedNode$
            .toArray()
            .toPromise();
        const pkgsToSave = rootResolvedNodes.concat(opts.localPackages);
        shrinkwrap_1.syncShrinkwrapWithManifest(opts.shrinkwrap, opts.pkg, pkgsToSave.map(resolvedNode => ({
            optional: resolvedNode.optional,
            dev: resolvedNode.dev,
            absolutePath: resolvedNode.absolutePath,
            name: resolvedNode.name,
            resolution: resolvedNode.resolution,
        })));
        const newShr = pnpm_shrinkwrap_1.prune(opts.shrinkwrap, opts.pkg);
        const waitq = [];
        waitq.push(removeOrphanPkgs_1.default({
            oldShrinkwrap: opts.privateShrinkwrap,
            newShrinkwrap: newShr,
            prefix: opts.root,
            store: opts.storePath,
            storeIndex: opts.storeIndex,
            bin: opts.bin,
        }));
        let wantedRootResolvedNode$ = rootResolvedNode$.filter(dep => !opts.skipped.has(dep.pkgId));
        if (opts.production) {
            wantedRootResolvedNode$ = wantedRootResolvedNode$.filter(dep => !dep.dev);
        }
        if (!opts.optional) {
            wantedRootResolvedNode$ = wantedRootResolvedNode$.filter(dep => !dep.optional);
        }
        waitq.push(wantedRootResolvedNode$.mergeMap(resolvedNode => {
            return Rx.Observable.fromPromise(symlinkDependencyTo(resolvedNode, opts.baseNodeModules))
                .map(symlinkingResult => {
                if (!symlinkingResult.reused) {
                    pnpm_logger_1.rootLogger.info({
                        added: {
                            id: resolvedNode.pkgId,
                            name: resolvedNode.name,
                            version: resolvedNode.version,
                            dependencyType: resolvedNode.dev && 'dev' || resolvedNode.optional && 'optional' || 'prod',
                        },
                    });
                }
                logInstallStatus_1.default({
                    status: 'installed',
                    pkgId: resolvedNode.pkgId,
                });
            });
        })
            .toPromise());
        waitq.push(resolvedNode$
            .map(resolvedNode => [resolvedNode.absolutePath, resolvedNode])
            .toArray()
            .map(R.fromPairs)
            .toPromise());
        const resolvedNodesMap = (yield Promise.all(waitq))[2];
        yield linkBins_1.default(opts.baseNodeModules, opts.bin);
        return {
            resolvedNodesMap,
            shrinkwrap: newShr,
            updatedPkgsAbsolutePaths,
        };
    });
}
exports.default = default_1;
function filterShrinkwrap(shr, opts) {
    let pairs = R.toPairs(shr.packages)
        .filter(pair => !opts.skipped.has(pair[1].id || dp.resolve(shr.registry, pair[0])));
    if (opts.noDev) {
        pairs = pairs.filter(pair => !pair[1].dev);
    }
    if (opts.noOptional) {
        pairs = pairs.filter(pair => !pair[1].optional);
    }
    return {
        shrinkwrapVersion: shr.shrinkwrapVersion,
        registry: shr.registry,
        specifiers: shr.specifiers,
        packages: R.fromPairs(pairs),
    };
}
function linkNewPackages(privateShrinkwrap, resolvedPkg$, opts, registry) {
    let copy = false;
    const prevPackages = privateShrinkwrap.packages || {};
    const outOfDateResolvedPkg$ = resolvedPkg$
        .filter(resolvedPkg => {
        if (!resolvedPkg.node.installable)
            return false;
        // TODO: what if the registries differ?
        if (opts.force || !prevPackages[resolvedPkg.dependencyPath]) {
            return true;
        }
        // add subdependencies that have been updated
        // TODO: no need to relink everything. Can be relinked only what was changed
        return !R.equals(prevPackages[resolvedPkg.dependencyPath].dependencies, resolvedPkg.snapshot.dependencies) ||
            !R.equals(prevPackages[resolvedPkg.dependencyPath].optionalDependencies, resolvedPkg.snapshot.optionalDependencies);
    })
        .shareReplay(Infinity);
    const pkgWithLinkedModules$ = outOfDateResolvedPkg$
        .mergeMap(resolvedPkg => {
        const wantedDependencies = resolvedPkg.dependencies.concat(opts.optional ? resolvedPkg.optionalDependencies : []);
        return linkModules(resolvedPkg.node, wantedDependencies)
            .mapTo({
            resolvedPkg,
            dependenciesWithBins: wantedDependencies.filter(pkg => pkg.hasBins),
        });
    })
        .shareReplay(Infinity);
    const pkgWithLinkedContent$ = outOfDateResolvedPkg$
        .mergeMap(resolvedPkg => {
        const linkPkgContent$ = copy
            ? Rx.Observable.fromPromise(linkPkgToAbsPath(copyPkg, resolvedPkg.node, opts))
            : Rx.Observable.fromPromise(linkPkgToAbsPath(linkPkg, resolvedPkg.node, opts))
                .catch(err => {
                if (!err.message.startsWith('EXDEV: cross-device link not permitted'))
                    throw err;
                copy = true;
                pnpm_logger_1.default.warn(err.message);
                pnpm_logger_1.default.info('Falling back to copying packages from store');
                return Rx.Observable.fromPromise(linkPkgToAbsPath(copyPkg, resolvedPkg.node, opts));
            });
        if (resolvedPkg.node.hasBundledDependencies) {
            // link the bundled dependencies` bins
            return linkPkgContent$
                .mergeMap(() => {
                const binPath = path.join(resolvedPkg.node.hardlinkedLocation, 'node_modules', '.bin');
                const bundledModules = path.join(resolvedPkg.node.hardlinkedLocation, 'node_modules');
                return Rx.Observable.fromPromise(linkBins_1.default(bundledModules, binPath));
            })
                .mapTo({
                resolvedPkg,
            });
        }
        return linkPkgContent$
            .mapTo({
            resolvedPkg,
        });
    })
        .shareReplay(Infinity);
    return pkgWithLinkedContent$.mergeMap(linkedPkg => {
        return pkgWithLinkedModules$
            .single(withLinkedModules => withLinkedModules.resolvedPkg.node.absolutePath === linkedPkg.resolvedPkg.node.absolutePath)
            .mergeMap(linkedPkg => {
            if (!linkedPkg.dependenciesWithBins.length) {
                return Rx.Observable.of(linkedPkg.resolvedPkg);
            }
            return Rx.Observable.from(linkedPkg.dependenciesWithBins)
                .mergeMap(depWithBins => {
                return pkgWithLinkedContent$
                    .map(_ => _.resolvedPkg.node)
                    .filter(resolvedNode => resolvedNode.absolutePath === depWithBins.absolutePath)
                    .concat(Rx.Observable.of(depWithBins))
                    .take(1);
            })
                .mergeMap(resolvedNode => {
                return _linkBins(linkedPkg.resolvedPkg.node, resolvedNode);
            })
                .last();
        })
            .mapTo(linkedPkg.resolvedPkg.node.absolutePath);
    });
}
function linkPkgToAbsPath(linkPkg, pkg, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const fetchResult = yield pkg.fetchingFiles;
        if (pkg.independent)
            return Rx.Observable.of(undefined);
        return linkPkg(fetchResult, pkg, opts);
    });
}
function _linkBins(pkg, dependency) {
    const binPath = path.join(pkg.hardlinkedLocation, 'node_modules', '.bin');
    if (!dependency.installable)
        return Rx.Observable.empty();
    return Rx.Observable.fromPromise(linkBins_1.linkPkgBins(path.join(pkg.modules, dependency.name), binPath));
}
function linkModules(pkg, deps) {
    if (pkg.independent)
        return Rx.Observable.of(undefined);
    return Rx.Observable.fromPromise(Promise.all(deps
        .filter(child => child.installable)
        .map(child => symlinkDependencyTo(child, pkg.modules))));
}
function linkPkg(fetchResult, dependency, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgJsonPath = path.join(dependency.hardlinkedLocation, 'package.json');
        if (fetchResult.isNew || opts.force || !(yield exists(pkgJsonPath)) || !(yield pkgLinkedToStore(pkgJsonPath, dependency))) {
            yield linkIndexedDir_1.default(dependency.path, dependency.hardlinkedLocation, fetchResult.index);
        }
    });
}
function copyPkg(fetchResult, dependency, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const newlyFetched = yield dependency.fetchingFiles;
        const pkgJsonPath = path.join(dependency.hardlinkedLocation, 'package.json');
        if (newlyFetched || opts.force || !(yield exists(pkgJsonPath))) {
            yield ncp(dependency.path, dependency.hardlinkedLocation);
        }
    });
}
function pkgLinkedToStore(pkgJsonPath, dependency) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgJsonPathInStore = path.join(dependency.path, 'package.json');
        if (yield isSameFile(pkgJsonPath, pkgJsonPathInStore))
            return true;
        pnpm_logger_1.default.info(`Relinking ${dependency.hardlinkedLocation} from the store`);
        return false;
    });
}
function isSameFile(file1, file2) {
    return __awaiter(this, void 0, void 0, function* () {
        const stats = yield Promise.all([fs.stat(file1), fs.stat(file2)]);
        return stats[0].ino === stats[1].ino;
    });
}
function symlinkDependencyTo(dependency, dest) {
    dest = path.join(dest, dependency.name);
    return symlinkDir(dependency.hardlinkedLocation, dest);
}
//# sourceMappingURL=index.js.map